
static char help[] = "Solve a small system and a large system through preloading\n\
  Input arguments are:\n\
   -fA Amat -fP Pmat -fG Gmat \n\n";
#include <petscksp.h>
#include "petscsys.h"

/* ATTENTION: this is the example used in the Profiling chaper of the PETSc manual,
   where we referenced its profiling stages, preloading and output etc.
   When you modify it, please make sure it is still consistent with the manual.
 */
int main(int argc,char **args)
{
  PetscErrorCode    ierr;
  Vec               x,b;
  Mat               A, P, G;           /* linear system matrix */
  KSP               ksp;         /* Krylov subspace method context */
  char              file[3][PETSC_MAX_PATH_LEN];
  PetscViewer       viewerA, viewerP, viewerG;      /* viewer */
  PetscBool         flg;
  PetscReal *       coords;

  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
  ierr = PetscOptionsGetString(NULL,NULL,"-fA",file[0],sizeof(file[0]),&flg);CHKERRQ(ierr);
  ierr = PetscOptionsGetString(NULL,NULL,"-fP",file[1],sizeof(file[1]),&flg);CHKERRQ(ierr);
  ierr = PetscOptionsGetString(NULL,NULL,"-fG",file[2],sizeof(file[1]),&flg);CHKERRQ(ierr);

  /* open binary file. Note that we use FILE_MODE_READ to indicate reading from this file */
  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[0],FILE_MODE_READ,&viewerA);CHKERRQ(ierr);
  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[1],FILE_MODE_READ,&viewerP);CHKERRQ(ierr);
  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[2],FILE_MODE_READ,&viewerG);CHKERRQ(ierr);

  /* load the matrix and vector; then destroy the viewer */
  ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
  ierr = MatCreate(PETSC_COMM_WORLD,&P);CHKERRQ(ierr);
  ierr = MatCreate(PETSC_COMM_WORLD,&G);CHKERRQ(ierr);
  ierr = MatSetFromOptions(A);CHKERRQ(ierr);
  ierr = MatSetFromOptions(P);CHKERRQ(ierr);
  ierr = MatLoad(A,viewerA);CHKERRQ(ierr);
  ierr = MatLoad(P,viewerP);CHKERRQ(ierr);
  ierr = MatLoad(G,viewerG);CHKERRQ(ierr);

  ierr = MatCreateVecs(A,&b,NULL);CHKERRQ(ierr);
  ierr = VecSet(b,1.0);CHKERRQ(ierr);
  ierr = VecDuplicate(b, &x);CHKERRQ(ierr);

  // Random coords
  PetscInt    n, dim=2;
  PetscRandom r;
  PetscRandomCreate(PETSC_COMM_WORLD, &r);
  VecGetSize(b, &n);
  PetscMalloc1(dim*n,&coords);
  for(int i=0; i<dim*n;++i)
  {
	  PetscReal val;
	  PetscRandomGetValueReal(r, &val);
	  coords[i] = val;
	  
  }

  ierr = PetscViewerDestroy(&viewerA);CHKERRQ(ierr);
  ierr = PetscViewerDestroy(&viewerP);CHKERRQ(ierr);
  ierr = PetscViewerDestroy(&viewerG);CHKERRQ(ierr);

  ierr = KSPCreate(PETSC_COMM_WORLD,&ksp);CHKERRQ(ierr);
  ierr = KSPSetOperators(ksp,A,P);CHKERRQ(ierr);
  ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
  PC pc;
  KSPGetPC(ksp, &pc);
  PCSetCoordinates(pc, dim, n, coords);
  PCHYPRESetDiscreteGradient(pc, G);

  ierr = KSPSolve(ksp,b,x);CHKERRQ(ierr);

  ierr = KSPDestroy(&ksp);CHKERRQ(ierr);
  ierr = MatDestroy(&A);CHKERRQ(ierr);
  ierr = MatDestroy(&P);CHKERRQ(ierr);
  ierr = MatDestroy(&G);CHKERRQ(ierr);
  ierr = VecDestroy(&x);CHKERRQ(ierr);
  ierr = VecDestroy(&b);CHKERRQ(ierr);

  ierr = PetscFinalize();
  return ierr;
}

